//
// file :         readthread.cpp
//
// description :  reading thread
//
// project :      TANGO Device Server
//
// $Author: graziano $ 
//
// $Revision: 1.3 $
//
// $Log: readthread.cpp,v $
// Revision 1.3  2009/10/21 10:16:42  graziano
// fixed loss of configuration
//
// Revision 1.2  2009/10/02 15:30:59  graziano
// improved precision of scan period
//
// Revision 1.1  2009/09/22 12:10:18  graziano
// first commit
//
// Revision 1.6  2009/07/23 09:21:11  graziano
// added scheduler
//
// Revision 1.5  2009/06/24 11:56:06  graziano
// log temperature on files,
// fixed bug in reconnection
//
// Revision 1.4  2009/06/24 07:49:32  graziano
// renamed attributes
//
// Revision 1.3  2009/06/22 10:18:13  graziano
// added log
//
// Revision 1.2  2009/06/22 09:07:51  graziano
// development
//
// Revision 1.1  2009/06/18 07:16:53  graziano
// first commit
//
//
//
//
// copyleft :   Sincrotrone Trieste S.C.p.A.
//              Strada Statale 14 - km 163,5 in AREA Science Park
//              34012 Basovizza, Trieste ITALY
//

#include "readthread.h"
static const char __FILE__rev[] = __FILE__ " $Revision: 1.3 $";

//+------------------------------------------------------------------
//
//	method:			readthread::readthread()
//
//	description:	readthread constructor
//
//-------------------------------------------------------------------
readthread::readthread(Keithley2701_ns::Keithley2701 *s):Tango::LogAdapter(s)
{ 
	DEBUG_STREAM << "readthread::readthread(): constructor... :" << __FILE__rev << endl;
	mydevice = s;
}

//+------------------------------------------------------------------
//
//	method:			readthread::~readthread()
//
//	description:	readthread destructor
//
//-------------------------------------------------------------------
readthread::~readthread()
{
	DEBUG_STREAM << "readthread::~readthread(): destructor... !" << endl;
	mydevice->terminated = true;
}

//+------------------------------------------------------------------
//
//	method:			readthread::run()
//
//	description:	Run
//
//-------------------------------------------------------------------
void readthread::run(void *) 
{
	INFO_STREAM << "readthread::run(): running... !" << endl;	
	int pausesec,pausenano,pausesec_dfl,pausenano_dfl,
	    cnt_err = 0,
		max_cnt_err = 3,
		ret;
	struct timeval start;
	struct timeval now;
	double diff_usec;
		
	pausesec = pausesec_dfl = (int)(mydevice->th_sleep/1000);
	pausenano = pausenano_dfl = (int)(mydevice->th_sleep-pausesec*1000)*1000000;
	usleep(350000);

	while (!mydevice->abortflag) 
	{	
		gettimeofday(&start,NULL);
		if(mydevice->scanning)
			ret = mydevice->Scan();
		gettimeofday(&now,NULL);
		diff_usec = (double)diff_time(start, now);
		if(ret == -1)	//lost configuration
		{
			stringstream stat;
			stat << mydevice->get_status() << " | Lost configuration at " << (long)now.tv_sec << " sec." << ends;
			mydevice->set_status(stat.str());
			mydevice->Configure();
		}

/*		Tango::MultiAttribute *attr_list = mydevice->get_device_attr();
		Tango::Attribute &attr = attr_list->get_attr_by_name ("Temp0"); 
		Tango::UserDefaultAttrProp	attr_prop;
		attr_prop.set_unit("C");
		attr_prop.set_standard_unit("1.0");
		attr_prop.set_display_unit("C");
		attr.set_default_properties(attr_prop);*/
	
		if (ret < 0) 
			cnt_err++;
		else
			cnt_err = 0;	
		if (cnt_err > max_cnt_err) {
			pausesec = 5;
			pausenano = 0;
		}
		else {
			pausesec = pausesec_dfl;
			pausenano = pausenano_dfl;
		}
		//omni_thread::sleep(pausesec,pausenano);
		//sleep at least 100 ms
		double sleep_ms = (mydevice->th_sleep - diff_usec/1000) > 100 ? (mydevice->th_sleep - diff_usec/1000) : 100;
		DEBUG_STREAM << "readthread::run(): measuring time="<<diff_usec/1000<<"ms, period="<<mydevice->th_sleep<<"ms, going to sleep "<<sleep_ms<<"ms"<<endl; 
		//abort_sleep(sleep_ms/1000);
		abort_sleep_precise(start,mydevice->th_sleep/1000);		
	}
	INFO_STREAM << "readthread::run(): exit!!!" << endl;
}

//+------------------------------------------------------------------
//
//	method:			readthread::abort_sleep
//
//	description:	Resume from sleep if abort_flag set (sec.)
//
//-------------------------------------------------------------------
void readthread::abort_sleep(double time) 
{		
	
	for (int i = 0; i < (time/0.2); i++) {
		if (mydevice->abortflag)
			break;
		omni_thread::sleep(0,200000000);
	}

}

//+------------------------------------------------------------------
//
//	method:			readthread::abort_sleep_precise
//
//	description:	Resume from sleep if abort_flag set (sec.)
//
//-------------------------------------------------------------------
void readthread::abort_sleep_precise(timeval from, double time)
{
	timeval now;
	double diff_usec;
	double sleep_ms = (mydevice->th_sleep - diff_usec/1000) > 100 ? (mydevice->th_sleep - diff_usec/1000) : 100;
	for (int i = 0; i < (time/0.2); i++) 
	{
		if (mydevice->abortflag)
			break;
		gettimeofday(&now,NULL);
		diff_usec = (double)diff_time(from, now);
		if(time - diff_usec/1000000 > 0.200)
		{
			omni_thread::sleep(0,200000000);
		}
		else if (time - diff_usec/1000000 <= 0.001)
		{
			return;
		}
		else
		{
			unsigned long sleep_ns = (unsigned long)(time*1000000000 - diff_usec*1000) - 1000000;	//test: -1ms
			DEBUG_STREAM << "abort_sleep_precise: nanosleeping -> " << sleep_ns << endl;
			omni_thread::sleep(0,sleep_ns);
		}
	}

}

//+------------------------------------------------------------------
//
//	method:			readthread::diff_time
//
//	description:	Difference in usec
//
//-------------------------------------------------------------------
unsigned long readthread::diff_time(struct timeval start, struct timeval end)
{
	return ((end.tv_sec-start.tv_sec)*1000000+(end.tv_usec-start.tv_usec));
}


